
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string.h>

using namespace std;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

const int MAX_STRLEN = 256;

class StringClass
{
public:

  StringClass() {  capacity = MAX_STRLEN; a = new char[capacity]; length = 0; }
  StringClass(const char *s);

  ~StringClass() { cout << "destructor!!\n"; delete [] a; }     // 1!
  StringClass(StringClass &);                                   // 2!
  StringClass & operator=(const StringClass &);                 // 3!

  void print();

  char *a;        // dynamic array for string chars
  int capacity;   // max size of array
  int length;     // actual number of chars used

};

//----------------------------------------------------------------------------

// constructor which takes string literal

StringClass::StringClass(const char *s)
{
 capacity = MAX_STRLEN; 
 a = new char[capacity]; 
 length = strlen(s); 

 for (int i = 0; i < length; i++)
   a[i] = s[i];
}

//----------------------------------------------------------------------------

// copy constructor

StringClass::StringClass(StringClass & s)
{
 capacity = s.capacity; 
 length = s.length;

 // shallow copy
 // a = s.a;    

 // deep copy

 a = new char[capacity];
 for (int i = 0; i < length; i++)
   a[i] = s.a[i];
}

//----------------------------------------------------------------------------

// assignment operator

StringClass & StringClass::operator=(const StringClass &rtSide)
{
  if (this != &rtSide) {

    capacity = rtSide.capacity;
    length = rtSide.length;
    delete [] a;  

    a = new char[capacity];
    for (int i = 0; i < length; i++)
      a[i] = rtSide.a[i];
  }

  return *this;
}

//----------------------------------------------------------------------------

void StringClass::print()
{
  for (int i = 0; i < length; i++)
    cout << a[i];
  cout << endl;
}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

int main()
{
  /*
  StringClass s("boolaboola");
  StringClass t("wahoo");
  StringClass u(s);

  s.print();
  t.print();
  u.print();

  // destructor on u unhappy because u.a was not new'd

  delete &s;  // not good because we didn't create s with new
  */

  StringClass *s = new StringClass("boolaboola");
  StringClass *t = new StringClass("wahoo");
  StringClass *u = new StringClass(*s);

  s->print();
  t->print();
  u->print();

  delete s;  // fine because s was created with new

  s = new StringClass("what?");
  s->print();

  //  u->print();   // try this with and without shallow copy

  return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------



